
/* Clock Component.h
	Standard calls and values for a clock-component.

	by Jim Batson, Peter Hoddie, Bruce Leak, David Van Brink
	Copyright 1990 Apple Computer, Inc. All rights reserved.
*/

#ifndef _ClockComponent_
#define _ClockComponent_

#include "Components.h"
#include "Timebases.h"

#define clockComponentType 		'clok'
#define systemTickClock 		'tick'			/* subtype: 60ths since boot		*/
#define systemSecondClock		'seco'			/* subtype: seconds since 1904		*/
#define systemMillisecondClock	'mill'			/* subtype: 1000ths since boot		*/
#define systemMicrosecondClock	'micr'			/* subtype: 1000000ths since boot	*/

enum {
	kClockRateIsLinear = 1,
	kClockImplementsCallBacks = 2
};

typedef enum
	{
	kClockGetClockTimeSelect = 1,
	kClockSetClockTimeSelect = 2,
	kClockNewCallBackSelect,
	kClockDisposeCallBackSelect,
	kClockCallMeWhenSelect,
	kClockCancelCallBackSelect,
	kClockRateChangedSelect,
	kClockTimeChangedSelect,
	kClockSetTimeBaseSelect
	} ClockComponentSelectors;

pascal ComponentResult GetClockTime(ComponentInstance aClock, TimeRecord *out)
		= {0x2f3c,4,kClockGetClockTimeSelect,0x7000,0xa82a};

pascal QTCallBack ClockNewCallBack(ComponentInstance aClock, TimeBase tb, short callBackType)
		= {0x2f3c,6,kClockNewCallBackSelect,0x7000,0xa82a};

pascal ComponentResult ClockDisposeCallBack(ComponentInstance aClock, QTCallBack cb)
		= {0x2f3c,4,kClockDisposeCallBackSelect,0x7000,0xa82a};

pascal ComponentResult ClockCallMeWhen(ComponentInstance clock, QTCallBack cb,
					long param1, long param2, long param3)
		= {0x2f3c,16,kClockCallMeWhenSelect,0x7000,0xa82a};

pascal ComponentResult ClockCancelCallBack(ComponentInstance clock, QTCallBack cb)
		= {0x2f3c,4,kClockCancelCallBackSelect,0x7000,0xa82a};

pascal ComponentResult ClockRateChanged(ComponentInstance clock, QTCallBack cb)
		= {0x2f3c,4,kClockRateChangedSelect,0x7000,0xa82a};

pascal ComponentResult ClockTimeChanged(ComponentInstance clock, QTCallBack cb)
		= {0x2f3c,4,kClockTimeChangedSelect,0x7000,0xa82a};

pascal ComponentResult ClockSetTimeBase(ComponentInstance clock, TimeBase tb)
		= {0x2f3c,4,kClockSetTimeBaseSelect,0x7000,0xa82a};

#endif _ClockComponent_
