/**************************************************
*
* Sample Movie Edit controls
*
*	This program is an extension of mini player 3
*	and demonstrates the editing features of the standard controller
*
*	Sample.h has definitions and equates
*	Main.c contains the general Mac application stuff
*	Movie Stuff.c has most of the interesting movie code
*
*	Interesting things to look at:
*	Enabling editing for the standard controller
*	Enabling the edit menu items
*	Handling the undo, cut, copy and paste events
*
* 	Rich Williams 7/91
*	Updated to MPW/Think 5.0/DSG interfaces 9/91 Chris Thorman
*
***************************************************/

/**************************************************
*
* General Info
*
***************************************************/
#include "Movies.h"
#include "Mini Edit.h"

#include <Memory.h>
#include <Palettes.h>
#include <Fonts.h>
#include <OSEvents.h>
#include <ToolUtils.h>
#include <Desk.h>
#include <Resources.h>
#include <Sound.h>
#include <GestaltEqu.h>

extern	short	numOpenMovies;
extern	MovieInstance	*activeMovie;		/* Pointer to the front movie */


Boolean	exitFlag;
Boolean hasWNE;

/**************************************************
*
*  various globals
*
***************************************************/

extern	short	theModifiers;					/* Used to keep the event modifiers */
Rect		dragRect;
MenuHandle	appleMenu, fileMenu, editMenu;

MovieInstance	movieList[maxMovies];		/* Array of open movies */


/**************************************************
***************************************************
*
* Initialization Routines
*
* All of the stuff we call to get things started
*
***************************************************
***************************************************/

/**************************************************
*
* InitMacintosh() Initialize all the managers & memory
*
***************************************************/
void InitMacintosh()

{
	MaxApplZone();
	InitGraf(&qd.thePort);
	InitFonts();
	FlushEvents(everyEvent, 0);
	InitWindows();
	InitMenus();
	TEInit();
	InitDialogs(0L);
	InitCursor();

}

/**************************************************
*
* SetUpMenus()
*
* Reads in the menu resources and installs them
*
*
***************************************************/
void SetUpMenus()

{
	InsertMenu(appleMenu = GetMenu(appleID), 0);
	InsertMenu(fileMenu = GetMenu(fileID), 0);
	InsertMenu(editMenu = GetMenu(editID), 0);
	DrawMenuBar();
	AddResMenu(appleMenu, 'DRVR');
	
}

/**************************************************
*
* SetUpWindows()
*
* INits stuff for windows
*
***************************************************/
void SetUpWindows()

{	
	/* Create the window for the picture */	
	dragRect = qd.screenBits.bounds;

}


/**************************************************
***************************************************
*
*	Routines for handling menus.
*
***************************************************
***************************************************/

/**************************************************
*
*  AdjustMenus()
*
*	Enable or disable the items in the Edit menu if a DA window
*	comes up or goes away. Our application doesn't do anything with
*	the Edit menu.
*
***************************************************/
void AdjustMenus()
{
	register WindowPeek wp = (WindowPeek) FrontWindow();
	short kind = wp ? wp->windowKind : 0;
	Boolean DA = kind < 0;
	long	controllerFlags = 0;
	OSErr	theErr;
	
	if(DA)					/* Enable the edit menu if it is a DA */
	{
		EnableItem(editMenu, undoItem);
		EnableItem(editMenu, cutItem);
		EnableItem(editMenu, copyItem);
		EnableItem(editMenu, pasteItem);
		EnableItem(editMenu, clearItem);
		DisableItem(editMenu, selectAllItem);
		EnableItem(fileMenu, closeItem);
		DisableItem(fileMenu, saveItem);
	}
	else 
	{
		if (activeMovie)			/* Is there a movie in the front window? */
		{
			if ( MCGetControllerInfo(activeMovie->movieController,&controllerFlags) == 0 ) {
				enable(fileMenu, saveItem,HasMovieChanged(activeMovie->movie));
				EnableItem(fileMenu, closeItem);
				EnableItem(editMenu, selectAllItem);
			}

		}
		else
		{
			DisableItem(fileMenu, saveItem);
			DisableItem(fileMenu, closeItem);
			DisableItem(editMenu, selectAllItem);
		}

		enable(editMenu,undoItem,controllerFlags & mcInfoUndoAvailable);
		enable(editMenu,cutItem,controllerFlags & mcInfoCutAvailable);
		enable(editMenu,copyItem,controllerFlags & mcInfoCopyAvailable);
		enable(editMenu,pasteItem,controllerFlags & mcInfoPasteAvailable);
		enable(editMenu,clearItem,controllerFlags & mcInfoClearAvailable);
		
		enable(fileMenu, openItem, numOpenMovies<maxMovies);
		enable(fileMenu, closeItem,numOpenMovies);
	}
	

}

static
void enable(MenuHandle menu, short item, Boolean ok)
{
	if (ok)
		EnableItem(menu, item);
	else
		DisableItem(menu, item);
}

/**************************************************
*
*  HandleMenu (mSelect)
*
*	Handle the menu selection. mSelect is what MenuSelect() and
*	MenuKey() return: the high word is the menu ID, the low word
*	is the menu item
*
***************************************************/
void HandleMenu (mSelect)

long	mSelect;

{
	short			menuID = HiWord(mSelect);
	short			menuItem = LoWord(mSelect);
	Str255		name;
	GrafPtr		savePort;
	WindowPeek	frontWindow;
	
	switch (menuID)
	{
		case	appleID:				/* The Apple menu */
			switch (menuItem)
			{
				case	aboutItem:
					DoAboutBox();		/* Display the About Box */
					break;
					
				default:				/* It's a DA */
					GetPort(&savePort);
					GetItem(appleMenu, menuItem, name);
					OpenDeskAcc(name);
					SetPort(savePort);
					break;
			}
			break;
	
	  	case	fileID:					/* The file menu */
			switch (menuItem)
		  	{
		  		case	openItem:
		  			DoOpen();			/* Open a movie */
					break;
  				  			
		  		case	closeItem:
					if ((frontWindow = (WindowPeek) FrontWindow()) == 0L)
			  			break;
			  
					if (frontWindow->windowKind < 0)
			  			CloseDeskAcc(frontWindow->windowKind);
					else
						/* It's the movie window ( the only one we've got ) */
						/* Close it & clean up the movie stuff */
						CloseEm( (WindowPtr) frontWindow);

					break;

				case	saveItem:		/* save the file */
					SaveTheMovie(activeMovie);
					break;

									  				  	
				case	quitItem:		/* Pack up and go home */
					exitFlag = true;
					break;
			}
			break;
  				
		case	editID:
			/* DA edit? */
			if (!SystemEdit(menuItem-1))
				if(activeMovie)
		  			DoMovieEdit(activeMovie, menuItem);		/* Handle the movie edit */
			break;
	  }
}

/**************************************************
*
*  CloseEm(w:WindowPtr) Closes window w 
*	if w is a movie window, clean up the movie and the player
*
***************************************************/
void CloseEm(w)

WindowPtr w;

{
	MovieInstance	*theMovie;
	
	HideWindow(w);
	theMovie = WhichMovieWindow(w);	/* Is it a movie window */
	if (theMovie)	/* If its the movie window, throw out the movie */
	{
		CleanUpMovie(theMovie);
	}
}


/**************************************************
*
*  CloseAllWindows() 
*	Throws out all windows & movies
*
***************************************************/
void CloseAllWindows()

{
	short	movieCount;
	
	/* Throw out all of the movies */
	for (movieCount = 0;movieCount<maxMovies;movieCount++)
		CleanUpMovie(&movieList[movieCount]);

	/* Put any other windows here */
}

/**************************************************
*
*  DoAboutBox() Draws and displays the about box
*
***************************************************/
void DoAboutBox()

{
	DialogPtr	theDialog;		/* Stuff for the about box */
	short			itemHit, io;
	Handle		tempHandle, soundHandle;
	Rect		box;
	
	soundHandle = GetResource('snd ',meowID);
	theDialog = GetNewDialog(aboutDlgID,0L,(WindowPtr)-1L);
	
	/* Highlight the default button */
	SetPort(theDialog);
	GetDItem(theDialog,1,&itemHit,&tempHandle,&box);
	PenSize(3,3);
	InsetRect(&box,-4,-4);
	FrameRoundRect(&box,16,16);
	
	/* Wait until the button is pressed */
	do
		ModalDialog(0l,&itemHit);
	while (itemHit != 1);
	
	/* Button pressed, now meow and go home */
	io = SndPlay(0L,soundHandle,0);
	ReleaseResource(soundHandle);
	DisposDialog(theDialog);
}

void
DoDoubleClick()
{
	AppFile 	apfile;
	short 		printIt,m,n;
	FSSpec		fsSpec;

	CountAppFiles(&printIt,&n);
	
	if(n>0) {
		for(m=0;m<n;m++){
			GetAppFiles(m+1,&apfile);
			FSMakeFSSpec(apfile.vRefNum,0L,apfile.fName,&fsSpec);
			if ( apfile.fType == 'MooV' ) 
				OpenTheMovie(&fsSpec);
			if ( printIt ) {


			}
			ClrAppFiles(m+1);
		}
	}
}



OSErr 
MissedAEParameters (AppleEvent *message)
{
	DescType typeCode;
	Size actualSize;
	OSErr err;

	err = AEGetAttributePtr(message, keyMissedKeywordAttr, typeWildCard,
			&typeCode, nil, 0L, &actualSize);
	if (err == errAEDescNotFound)
		return(noErr);
	return(err = noErr ? errAEEventNotHandled : err);
}

pascal OSErr 
OpenDocMessage(AppleEvent *message, AppleEvent *reply, long refcon)
{
	FSSpec fSpec;
	AEDescList docList;
	long index, itemsInList;
	Size actualSize;
	AEKeyword keywd;
	DescType typeCode;
	OSErr err;

	if ((err = AEGetParamDesc(message, keyDirectObject, typeAEList, &docList)) != noErr)
		return(err);
	if ((err = MissedAEParameters(message)) != noErr)
		return(err);
	if ((err = AECountItems(&docList, &itemsInList)) != noErr)
		return(err);

	for (index = 1; index <= itemsInList; index++) {
		if ((err = AEGetNthPtr(&docList, index, typeFSS, &keywd, &typeCode,
					(Ptr)&fSpec, sizeof(FSSpec), &actualSize)) != noErr)
				break;
		OpenTheMovie(&fSpec);
	}

	return(AEDisposeDesc(&docList));
}

pascal OSErr 
QuitAppMessage(AppleEvent *message, AppleEvent *reply, long refcon)
{
	FSSpec fSpec;
	AEDescList docList;
	long index, itemsInList;
	Size actualSize;
	AEKeyword keywd;
	DescType typeCode;
	OSErr err;

	if ((err = MissedAEParameters(message)) != noErr)
		return(err);

	exitFlag = true;

	return(0);
}

/**************************************************
*
* DoOpen()
*
*	Brings up standard file dialog
*	and opens the movie if one has been selected
*
***************************************************/
void DoOpen(void)

{
	StandardFileReply		sfr;		/* New-style SF reply */
	SFTypeList	myTypeList = {'MooV'};
	
	StandardGetFilePreview(0, 1, myTypeList, &sfr);
	if (sfr.sfGood)
		OpenTheMovie(&sfr.sfFile);	

}

/**************************************************
***************************************************
*
*		The window routines
*
****************************************************
***************************************************/
 
/**************************************************
*
* WhichMovieWindow(w) 
*	Checks if window is for a movie
* 	and returns a pointer to it
*	or 0 if it isn't a movie's window
*
***************************************************/
MovieInstance* WhichMovieWindow(w)

WindowPtr	w;

{
	short	movieCount;
	
	/* Loop through looking for a match */
	for (movieCount = 0;movieCount<maxMovies;movieCount++)
		if (w == movieList[movieCount].window)
			break;
			
	if (movieCount<maxMovies)	/* Did we find one? */
		return(&movieList[movieCount]);
	
	return(0L);					/* Didn't find one */
}
		
/**************************************************
*
* IsMyWindow(w):Boolean 
*	Checks if w is a valid window and mine
*
***************************************************/
Boolean IsMyWindow(w)

WindowPtr	w;

{
	return( (w != 0L) && 
			 WhichMovieWindow(w)  );
}

	
/**************************************************
***************************************************
*
* Event handling stuff
*
**************************************************** 
***************************************************/

/**************************************************
*
* HandleMouseDown (theEvent)
*
*	Take care of mouseDown events.
*
***************************************************/
void HandleMouseDown(theEvent)

EventRecord	*theEvent;

{
	WindowPtr	theWindow;
	short			windowCode = FindWindow (theEvent->where, &theWindow);
	
	switch (windowCode)
	{
		case inSysWindow: 
	    	SystemClick (theEvent, theWindow);
	    	break;
	    
	  	case inMenuBar:
	  		AdjustMenus();
	    	HandleMenu(MenuSelect(theEvent->where));
	    	break;
	    
	  	case inDrag:
	  		if (IsMyWindow(theWindow))
	  	    	SelectWindow(theWindow);
	  	  		DragWindow(theWindow, theEvent->where, &dragRect);
	  	  	break;
	  	  
	  	case inContent:
	  		if (IsMyWindow(theWindow))
	  	  	{
	  	  		if (theWindow != FrontWindow())
	  	    		SelectWindow(theWindow);
	  	  		else
	  	    		MovieMouseDown(theWindow,theEvent->where,theEvent->modifiers); /* Track the mouse */
	  	  	}
	  		break;
	  	
		case inGoAway:
	  		if (IsMyWindow(theWindow) && TrackGoAway(theWindow, theEvent->where))
				CloseEm(theWindow);		/* Close the appropriate windows */
			break;
		
		break;
      }
}


/**************************************************
*
* HandleEvent()
*
*		The main event dispatcher. This routine should be called
*		repeatedly (it  handles only one event).
*
***************************************************/
void HandleEvent()

{
	short			ok;
	EventRecord	theEvent;
	WindowPtr	theWindow;
	
	MovieInstance	*theMovie;

	HiliteMenu(0);
	
	if (  hasWNE  ) {
		ok = WaitNextEvent (everyEvent, &theEvent,0,0);
	} else {
	
		SystemTask();
		ok = GetNextEvent (everyEvent, &theEvent);
	}
	/*!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
	!
	! Don't miss this!
	! MoviesTask is called from the main event loop to 
	! give extra time to the active movie
	! this is optional since CheckMovieControllers will update
	! all of the movies.  It also doesn't work too well yet.
	!
	!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!*/
	if(theEvent.what == nullEvent)
			MyMoviesTask();
	
	/*!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
	!
	! CheckMovieControllers is called from the main event loop to 
	! see if the event is for the player
	!
	!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!*/
	theModifiers = theEvent.modifiers;		/* We may need these in the filter */
	if (CheckMovieControllers(&theEvent))
		return;

	else if (ok)
	  switch (theEvent.what)
	  {		
		case mouseDown:
			HandleMouseDown(&theEvent);
			break;
			
		case keyDown: 
		case autoKey:
		    if ((theEvent.modifiers & cmdKey) != 0)
		    {
		      AdjustMenus();
			  HandleMenu(MenuKey((char) (theEvent.message & charCodeMask)));
			}
			break;
			
		case updateEvt:
			/* To update the window, make the player replay the last frame */
			theWindow = (WindowPtr) theEvent.message;	/* Get the window */

			theMovie = WhichMovieWindow(theWindow);			/* Is it a Movie window? */

			BeginUpdate(theWindow);			
			if(theMovie)
				DoMovieUpdate(theMovie);
			DrawControls(theWindow);
			EndUpdate(theWindow);
		    break;
		    
		case activateEvt:
			theMovie = WhichMovieWindow((WindowPtr) theEvent.message);		/* Is it a Movie window? */
			if (theMovie)
			{
				if (theEvent.modifiers & activeFlag)
					DoMovieActivate(theMovie);
				else
					DoMovieDeactivate(theMovie);
			}
			else		/* Some other window, make it redraw */
			{
				SetPort( (WindowPtr) theEvent.message );
				InvalRect(&((WindowPtr)theEvent.message)->portRect);	/* Force an update event */
			}
			break;
			
		case kHighLevelEvent:
			AEProcessAppleEvent(&theEvent);
			break;
	   }
}

/**************************************************
*
* main()
*
*	This is where everything happens
*
***************************************************/
main()

{	
	long	resp;

	InitMacintosh();		/* Initialize everything */
	SetUpMenus();
	SetUpWindows();
	SetUpMovies();
	exitFlag = false;

	hasWNE = NGetTrapAddress(0x60,ToolTrap) != NGetTrapAddress(0x9f,ToolTrap);

	if(	Gestalt(gestaltAppleEventsAttr, &resp) == 0 &&  resp != 0) {
		AEInstallEventHandler(kCoreEventClass, kAEOpenDocuments,
				(EventHandlerProcPtr)OpenDocMessage, 0, false);
		AEInstallEventHandler(kCoreEventClass, kAEQuitApplication,
				(EventHandlerProcPtr)QuitAppMessage, 0, false);
	}
	DoDoubleClick();
	while (!exitFlag )
		HandleEvent();
	CloseAllWindows();
	ExitToShell();
}
